<?php

use App\Models\Enums\ReportStatus;
use App\Models\Enums\StudyAccessFlags;
use App\Models\Enums\StudyLevelStatus;
use App\Models\Institute;
use App\Models\User;
use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    public function up(): void
    {
        Schema::create('studies', function (Blueprint $table) {
            $table->id();
            $table->string('orthanc_uid')->unique();
            $table->boolean('is_active')->default(true);
            $table->boolean('is_locked')->default(true);
            $table->unsignedTinyInteger('study_priority')->default(0);
            $table->string('patient_id')->nullable();
            $table->string('patient_uid')->nullable();
            $table->string('patient_name');
            $table->string('patient_sex')->nullable();
            $table->date('patient_birthdate')->nullable();

            $table->string('study_instance_uid')->unique();
            $table->string('study_id')->nullable();
            $table->string('institution_name')->nullable();
            $table->string('accession_number')->nullable();
            $table->string('study_description')->nullable();
            $table->string('body_part_examined')->nullable();
            $table->string('station_name')->nullable();
            $table->string('operators_name')->nullable();
            $table->string('manufacturer')->nullable();
            $table->string('manufacturer_model_name')->nullable();
            $table->string('software_versions')->nullable();

            $table->string('referring_physician_name')->nullable();
            $table->string('study_modality', 4)->nullable();
            $table->dateTime('study_date');
            $table->dateTime('receive_date');
            $table->dateTime('report_date')->nullable();
            $table->foreignIdFor(Institute::class)->constrained()->onDelete('cascade');
            $table->unsignedTinyInteger('study_status')->default(StudyLevelStatus::Pending->value);
            $table->unsignedTinyInteger('report_status')->default(ReportStatus::Pending->value);

            $table->unsignedSmallInteger('image_count')->nullable();
            $table->unsignedSmallInteger('series_count')->nullable();
            $table->unsignedInteger('disk_size')->nullable();

            $table->foreignIdFor(User::class, 'assigned_physician_id')->nullable()->constrained()->onDelete('set null');
            $table->foreignIdFor(User::class, 'interpreting_physician_id')->nullable()->constrained()->onDelete('set null');
            $table->foreignIdFor(User::class, 'referring_physician_id')->nullable()->constrained()->onDelete('set null');
            $table->foreignIdFor(User::class, 'reading_physician_id')->nullable()->constrained()->onDelete('set null');

            $table->unsignedTinyInteger('access_flags')->default(StudyAccessFlags::Forbidden->value);
            $table->string('access_password')->nullable();
            $table->timestamps();

            $table->index(['referring_physician_id', 'receive_date']);
            $table->index(['institute_id', 'receive_date']);
            $table->index(['institute_id', 'report_status', 'study_priority', 'receive_date']);
            $table->index(['assigned_physician_id', 'report_status', 'study_priority', 'receive_date']);
        });
    }

    public function down(): void
    {
        Schema::dropIfExists('studies');
    }
};
